@@ -24,11 +24,14 @@
 
 #include "core/pos.h"
 #include "core/move.h"
+#include "core/item.h"
 #include "rules/rules.h"
+
 #include "ui_promotiondialog.h"
 #include "gamemanager.h"
 
-#include <KGameTheme>
+#include <KgTheme>
+#include <KGameRenderer>
 #include <KDebug>
 #include <KDialog>
 
@@ -40,8 +43,6 @@
 #include <QtGui/QGraphicsView>
 #include <QtCore/qmath.h>
 
-#include "core/item.h"
-#include "core/renderer.h"
 
 using namespace Knights;
 
@@ -68,16 +69,19 @@ const QString blackLettersKey = QLatin1String ( "BlackLetters" );
 const QString whiteNumbersKey = QLatin1String ( "WhiteNumbers" );
 const QString blackNumbersKey = QLatin1String ( "BlackNumbers" );
 
-Board::Board ( QObject* parent ) : QGraphicsScene ( parent )
+Board::Board ( KgThemeProvider* provider, QObject* parent ) : QGraphicsScene ( parent ), 
+m_themeProvider(provider)
 {
-    renderer = new Renderer ( Settings::theme() );
+    renderer = new KGameRenderer ( m_themeProvider );
     m_background = 0;
     selectedPiece = 0;
+    draggedPiece = 0;
     Manager::self()->rules()->setGrid ( &m_grid );
     m_currentPlayer = White;
     updateTheme();
-    m_paused = false;
     m_dragActive = false;
+    
+    connect (provider, SIGNAL(currentThemeChanged(const KgTheme*)), SLOT(updateTheme()));
 }
 
 Board::~Board()
@@ -101,8 +105,9 @@ void Board::addPiece ( PieceType type, Color color, const Pos& pos )
 
 void Board::movePiece ( const Move& move )
 {
+    kDebug() << move;
     Move m = move;
-    if ( m.flag ( Move::Illegal ) ||  m.to() == m.from() || !m_grid.contains ( m.from() ) )
+    if ( ( m.flag ( Move::Illegal ) && !m.flag ( Move::Forced ) ) ||  m.to() == m.from() || !m_grid.contains ( m.from() ) )
     {
         kWarning() << "Invalid move:" << m;
         return;
@@ -172,13 +177,11 @@ void Board::movePiece ( const Move& move )
         }
     }
 
-    if ( m.flags() & Move::Castle )
+    foreach ( const Move& additionalMove, m.additionalMoves() )
     {
-        foreach ( const Move& additionalMove, m.additionalMoves() )
-        {
-            movePiece ( additionalMove );
-        }
+        movePiece ( additionalMove );
     }
+
     updateGraphics();
 }
 
@@ -215,7 +218,7 @@ void Board::addTiles()
 
 void Board::mousePressEvent ( QGraphicsSceneMouseEvent* e )
 {
-    if ( m_paused || !(m_playerColors & m_currentPlayer) )
+    if ( !Manager::self()->canLocalMove() )
     {
         // It is not the human player's turn
         e->ignore();
@@ -346,7 +349,7 @@ void Board::dropEvent ( QGraphicsSceneDragDropEvent* e )
 
 void Board::dragEnterEvent ( QGraphicsSceneDragDropEvent* e )
 {
-    e->accept();
+    e->setAccepted ( Manager::self()->canLocalMove() );
 }
 
 void Board::dragMoveEvent ( QGraphicsSceneDragDropEvent* e )
@@ -441,23 +444,21 @@ void Board::setPlayerColors ( Colors colors )
     populate();
 }
 
-void Board::changeCurrentPlayer()
-{
-    m_currentPlayer = oppositeColor ( m_currentPlayer );
-    if ( ( m_playerColors & m_currentPlayer ) && m_displayedPlayer != m_currentPlayer )
-    {
-        m_displayedPlayer = m_currentPlayer;
-        changeDisplayedPlayer();
-    }
-    emit activePlayerChanged ( m_currentPlayer );
-}
-
 void Board::setCurrentColor ( Color color )
 {
     m_currentPlayer = color;
-    if ( m_playerColors & color )
+    Color nextPlayer = m_displayedPlayer;
+    if ( ( ( m_playerColors & (Black|White) ) == (Black|White) ) && !Settings::flipBoard() )
     {
-        m_displayedPlayer = color;
+        nextPlayer = White;
+    }
+    else if ( m_playerColors & color )
+    {
+        nextPlayer = color;
+    }
+    if ( m_displayedPlayer != nextPlayer )
+    {
+        m_displayedPlayer = nextPlayer;
         changeDisplayedPlayer();
     }
     emit activePlayerChanged ( m_currentPlayer );
@@ -496,30 +497,8 @@ void Board::addMarker ( const Pos& pos, const QString& spriteKey )
     markers.insert ( pos, marker );
 }
 
-void Board::setPaused ( bool paused )
-{
-    m_paused = paused;
-}
-
-
 void Board::updateTheme()
 {
-    renderer->setTheme ( Settings::theme() );
-#if not defined WITH_KGR
-    // Using QGraphicsSvgItems, loading a new file and then resizing does not work
-    // instead, we have to delete every item and re-create it
-    foreach ( Piece* p, m_grid )
-    {
-        addPiece ( p->pieceType(), p->color(), m_grid.key ( p ) );
-        delete p;
-    }
-    // If the user is changing the theme, he/she probably already saw any current markers
-    qDeleteAll ( markers );
-    markers.clear();
-
-    qDeleteAll ( m_tiles );
-    m_tiles.clear();
-#endif
     delete m_background;
     if ( renderer->spriteExists ( backgroundKey ) )
     {
@@ -739,5 +718,6 @@ PieceType Board::getPromotedType()
     }
     return Queen;
 }
+
 #include "board.moc"
 // kate: indent-mode cstyle; space-indent on; indent-width 4; replace-tabs on;  replace-tabs on;  replace-tabs on;
